"use strict";
function genSequence(i) {
    return {
        [Symbol.iterator]: () => i[Symbol.iterator](),
        next: () => i[Symbol.iterator]().next(),
        map: (fn) => genSequence(map(fn, i)),
        filter: (fnFilter) => genSequence(filter(fnFilter, i)),
        reduce: (fnReduce, initValue) => {
            return reduce(fnReduce, initValue, i);
        },
        reduceToSequence: (fnReduce, initialValue) => {
            return genSequence(reduce(fnReduce, initialValue, i));
        },
        scan: (fnReduce, initValue) => {
            return genSequence(scan(i, fnReduce, initValue));
        },
        combine: (fn, j) => {
            return genSequence(combine(fn, i, j));
        },
        concat: (j) => {
            return genSequence(concat(i, j));
        },
        concatMap: (fn) => {
            return genSequence(concatMap(fn, i));
        },
        skip: (n) => {
            return genSequence(skip(n, i));
        },
        take: (n) => {
            return genSequence(take(n, i));
        },
        first: (fnFilter) => {
            return first(fnFilter, i);
        },
        toArray: () => [...i[Symbol.iterator]()],
        toIterable: () => {
            return toIterator(i);
        },
    };
}
exports.genSequence = genSequence;
// Naming Compatibility
exports.GenSequence = genSequence;
function* filter(fnFilter, i) {
    for (const v of i) {
        if (fnFilter(v)) {
            yield v;
        }
    }
}
exports.filter = filter;
function reduce(fnReduce, initialValue, i) {
    let index = 0;
    if (initialValue === undefined) {
        index = 1;
        const r = i[Symbol.iterator]().next();
        initialValue = r.value;
    }
    let prevValue = initialValue;
    for (const t of i) {
        const nextValue = fnReduce(prevValue, t, index);
        prevValue = nextValue;
        index += 1;
    }
    return prevValue;
}
exports.reduce = reduce;
function* scan(i, fnReduce, initValue) {
    let index = 0;
    if (initValue === undefined) {
        index = 1;
        const r = i[Symbol.iterator]().next();
        initValue = r.value;
        if (!r.done) {
            yield r.value;
        }
    }
    let prevValue = initValue;
    for (const t of i) {
        const nextValue = fnReduce(prevValue, t, index);
        yield nextValue;
        prevValue = nextValue;
        index += 1;
    }
}
exports.scan = scan;
function map(fnMap, i) {
    function* fn(fnMap, i) {
        for (const v of i) {
            yield fnMap(v);
        }
    }
    if (i !== undefined) {
        return fn(fnMap, i);
    }
    return function (i) {
        return fn(fnMap, i);
    };
}
exports.map = map;
/**
 * Combine two iterables together using fnMap function.
 */
function* combine(fnMap, i, j) {
    const jit = j[Symbol.iterator]();
    for (const r of i) {
        const s = jit.next().value;
        yield fnMap(r, s);
    }
}
exports.combine = combine;
/**
 * Concat two iterables together
 */
function* concat(i, j) {
    yield* i;
    yield* j;
}
exports.concat = concat;
function scanMap(accFn, init) {
    let acc = init;
    let first = true;
    return function (value) {
        if (first && acc === undefined) {
            first = false;
            acc = value;
            return acc;
        }
        acc = accFn(acc, value);
        return acc;
    };
}
exports.scanMap = scanMap;
function* skip(n, i) {
    let a = 0;
    for (const t of i) {
        if (a >= n) {
            yield t;
        }
        a += 1;
    }
}
exports.skip = skip;
function* take(n, i) {
    let a = 0;
    if (n) {
        for (const t of i) {
            if (a >= n) {
                break;
            }
            yield t;
            a += 1;
        }
    }
}
exports.take = take;
function* concatMap(fn, i) {
    for (const t of i) {
        for (const u of fn(t)) {
            yield u;
        }
    }
}
exports.concatMap = concatMap;
function first(fn, i) {
    fn = fn || (t => true);
    for (const t of i) {
        if (fn(t)) {
            return t;
        }
    }
    return undefined;
}
exports.first = first;
function* toIterator(i) {
    yield* i;
}
exports.toIterator = toIterator;
function* objectIterator(t) {
    const keys = new Set(Object.keys(t));
    for (const k in t) {
        if (keys.has(k)) {
            yield [k, t[k]];
        }
    }
}
exports.objectIterator = objectIterator;
function objectToSequence(t) {
    return genSequence(objectIterator(t));
}
exports.objectToSequence = objectToSequence;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = genSequence;
//# sourceMappingURL=GenSequence.js.map